/* $Id: gmin_main.c,v 3.2 1998/03/21 00:35:53 kas Exp $ */

/* Encapsulating routines for SA-algorithm */

/*
 *  Programs used in my diploma thesis.
 *  Copyright (C) 1995-1997  Jan "Yenya" Kasprzak <kas@fi.muni.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "config.h"
#ifdef HAVE_GETOPT_LONG
#include <getopt.h>
#else
#include "getopt.h"
#endif

#include "ff.h"
#include "pval.h"

static char *program_name;

static void usage();
static void generate_coordinates __P((potential_value *pvp));

#define LMIN_ACCURACY 1.0e-5

int main(argc,argv)
int argc;
char **argv;
{
	REAL vdw14 = -1.0, eel14 = -1.0, diel_lin = -1.0, diel_const = -1.0;
	REAL cutoff = -1.0;
	REAL temperature = 0.0, move_scale = 0.0, temp_factor = 0.0;
	REAL move_factor = -1.0;
	int max_accepted = 0, max_iterations = 0;
	int seed = 0;
	force_field *ffp;
	potential_value *pvp;
	char *outfile = NULL;
	char *infile = NULL;
#ifndef NONTRANSFORMED_ONLY
	char *ptmpl = NULL, *atmpl = NULL, *ttmpl = NULL;
	REAL lambda0 = -1, lambdaN=-1, lambda_factor = -1;
#endif
	int i, dim;

	program_name = *argv;

	while(1)
	{
		int c, opt_ind;
		static struct option long_opts[] = {
			{ "help", 0, 0, 'h' },
			{ "version", 0, 0, 'v' },
			{ "vdw14", 1, 0, 'w' },
			{ "eel14", 1, 0, 'e' },
			{ "diel-lin", 1, 0, 'l' },
			{ "diel-const", 1, 0, 'c' },
			{ "cutoff", 1, 0, 'C' },
			{ "seed", 1, 0, 's' },
			{ "outfile", 1, 0, 'o' },
			{ "begin-coords", 1, 0, 'b' },
			{ "temperature", 1, 0, 't' },
			{ "move-scale", 1, 0, 'm' },
			{ "move-factor", 1, 0, 'M' },
			{ "max-accepted", 1, 0, 'a' },
			{ "max-iterations", 1, 0, 'i' },
			{ "temp-factor", 1, 0, 'f' },
#ifndef NONTRANSFORMED_ONLY
			{ "start-lambda", 1, 0, 'L' },
			{ "end-lambda", 1, 0, 'E' },
			{ "lambda-factor", 1, 0, 'F' },
			{ "pair-template", 1, 0, 'P' },
			{ "angle-template", 1, 0, 'A' },
			{ "dih-template", 1, 0, 'D' },
#endif
			{ NULL }
		};

		if ((c=getopt_long(argc, argv, "hvw:e:l:c:s:o:b:t:m:a:i:f:--", long_opts,
			&opt_ind)) == -1)
			break;
		switch(c)
		{
		case 'h':
		case '?':
			usage();
		case 'v':
			printf("Version: $Id: gmin_main.c,v 3.2 1998/03/21 00:35:53 kas Exp $\n");
			return 0;
		case 'w':
			vdw14 = atof(optarg);
			break;
		case 'e':
			eel14 = atof(optarg);
			break;
		case 'l':
			diel_lin = atof(optarg);
			break;
		case 'c':
			diel_const = atof(optarg);
			break;
		case 'C':
			cutoff = atof(optarg);
			break;
		case 's':
			seed = atoi(optarg);
			break;
		case 'o':
			outfile = optarg;
			break;
		case 'b':
			infile = optarg;
			break;
		case 't':
			temperature = atof(optarg);
			break;
		case 'm':
			move_scale = atof(optarg);
			break;
		case 'M':
			move_factor = atof(optarg);
			break;
		case 'a':
			max_accepted = atoi(optarg);
			break;
		case 'i':
			max_iterations = atoi(optarg);
			break;
		case 'f':
			temp_factor = atof(optarg);
			break;
#ifndef NONTRANSFORMED_ONLY
		case 'P':
			ptmpl = optarg;
			break;
		case 'A':
			atmpl = optarg;
			break;
		case 'D':
			ttmpl = optarg;
			break;
		case 'L':
			lambda0 = atof(optarg);
			break;
		case 'E':
			lambdaN = atof(optarg);
			break;
		case 'F':
			lambda_factor = atof(optarg);
			break;
#endif
		default:
			usage();
			return -1;
		}
	}

	if (optind+1 > argc)
		usage();

	if ((ffp = prm2ff(argv[optind])) == NULL)
	{
		fprintf(stderr, "Cannot read force field from %s\n",
			argv[optind]);
		return 1;
	}

	setup_ff(ffp, vdw14, eel14, diel_lin, diel_const, cutoff);

	pvp = alloc_pv(ffp);

#ifdef LOCMIN
        if (infile)
                read_crd_file(pvp, infile);
        else {
		perror("Coordinate file required for local minimization.");
		exit(1);
	}

	minimize_l(pvp); 
#else

	if (seed == 0)
		seed = time(NULL);

	printf("Random number generator's seed set to %d\n", seed);
	srand(seed);

	if (infile)
		read_crd_file(pvp, infile);
	else
		generate_coordinates(pvp);

	setup_annealing(outfile, temperature, move_scale, move_factor, 
		 max_accepted, max_iterations, temp_factor
  #ifndef NONTRANSFORMED_ONLY
		, lambda0, lambdaN, lambda_factor,
		ptmpl, atmpl, ttmpl
  #endif
	);
	annealing(pvp);
#endif

	printf("Minimum is %f\n", pvp->value);

	return 0;
}

static void usage()
{
	fprintf(stderr, "Usage: %s [options] file.top\n"
		"Where [options] is zero or more from\n"
		"--help\tprints out this message\n"
		"--version\tprints out version of program\n"
		"--vdw14 number\tsets the scaling parameter for VdW 1-4 interactions\n"
		"--eel14 number\tsets the scaling parameter for E-static 1-4 interactions\n"
		"--diel-lin number\tsets the linear part of dielectric funtion\n"
		"--diel-const number\tsets the constant part of dielectric function\n"
		"--cutoff number\tsets the 1991 force field cutoff\n"
		"--seed number\tseed for the random number generator\n"
		"--outfile filename.crd\twhere coordinates of minimum state will be written\n"
		"--begin-coords filename.crd\tstarting state of the molecule\n"
		"--temperature number\tstarting temperature\n"
		"--move-scale number\thow far to move atoms during anneling\n"
		"--max-accepted number\tdecrease temperature after this # of accepted moves\n"
		"--max-iterations number\tdecrease temperature after this # of iterations\n"
		"--temp-factor number\thow to decrease temperature\n"
#ifndef NONTRANSFORMED_ONLY
		"--start-lambda number\tvalue of lambda to start with\n"
		"--end-lambda number\tfinal value of lambda\n"
		"--lambda-factor number\thow to decrease lambda\n"
		"--pair-template string\ttemplate for pair-interactions file\n"
		"--angle-template string\ttemplate for angle-interactions file\n"
		"--dih-template string\ttemplate for torsion-interactions file\n"
#endif
		,program_name);
	exit(1);
}

static void generate_coordinates(pvp)
potential_value *pvp;
{
        int i;

        for (i=0; i<pvp->ffp->natoms; i++)
        {
                pvp->x[i] = (REAL)pvp->ffp->natoms*((REAL)rand()/RAND_MAX);
                pvp->y[i] = (REAL)pvp->ffp->natoms*((REAL)rand()/RAND_MAX);
                pvp->z[i] = (REAL)pvp->ffp->natoms*((REAL)rand()/RAND_MAX);
        }
}

